#if !defined(RT_GDI_H)
#define RT_GDI_H
//----------------------------------------------------------
// rtgdi.h
//
// Some C++ helpers for GDI things.  WTL provides a much
// better implementation of these ideas, but introduces more
// complexity and another source code dependency.  These
// classes provide a some easy of dealing with Win32 in the
// book samples without being a large set of classes in a
// framework like WTL.  WTL is available in the Platform
// SDK as a free download from Microsoft's web site.
//

namespace rt
{

//-----------------------------------------------------------------------------
// c_get_dc
//
// Obtain a DC via ::GetDC in the c'tor and release it via ::ReleaseDC in
// the d'tor.  If this were a WTL or MFC application, I'd use their wrappers
// instead.
//
class c_get_dc
{
public:
    c_get_dc(HWND window)
        : m_window(window),
        m_dc(TWS(::GetDC(window)))
    {
    }
    ~c_get_dc()
    {
        ::ReleaseDC(m_window, m_dc);
    }
    operator HDC() const { return m_dc; }

private:
    HWND m_window;
    HDC m_dc;
};

//-----------------------------------------------------------------------------
// c_push_gdi<>
//
// Select a GDIOBJ into the device context in the c'tor and
// deselect the object out by selecting in the old object in the d'tor.
// The template parameter give the real type of the object, such
// as HBRUSH, HPEN, HFONT, etc.
//
template <typename T>
class c_push_gdi
{
public:
    c_push_gdi(HDC dc, T object)
        : m_dc(dc),
        m_old_object(static_cast<T>(::SelectObject(m_dc, object)))
    {
    }
    ~c_push_gdi()
    {
        ::SelectObject(m_dc, m_old_object);
    }
private:
    HDC m_dc;
    T m_old_object;
};

//-----------------------------------------------------------------------------
// c_create_font
//
// Simple resource class used to create an HFONT from a LOGFONT structure
// in its c'tor and delete the font in its d'tor.  WTL has a superior
// solution.
//
class c_create_font
{
public:
    c_create_font(const LOGFONT &lf)
        : m_font(0)
    {
        m_font = TWS(::CreateFontIndirect(&lf));
    }
    ~c_create_font()
    {
        ::DeleteObject(m_font);
    }

    operator HFONT() { return m_font; }

private:
    HFONT m_font;
};

}; // rt

#endif // RT_GDI_H
