#if !defined (RT_SKIN_H)
#define RT_SKIN_H

#include <math.h>

#define STRICT
#include <windows.h>
#include <basetsd.h>
#include <tchar.h>

#include <atlbase.h>

#include <D3DX8.h>
#include <DXErr8.h>

#include "D3DApp.h"
#include "D3DFont.h"
#include "D3DFile.h"
#include "D3DUtil.h"
#include "DXUtil.h"

#include "resource.h"
#include "rt/hr.h"

// Struct to store the current input state
struct UserInput
{
    bool bRotateUp;
    bool bRotateDown;
    bool bRotateLeft;
    bool bRotateRight;
};

struct s_mesh_bounds
{
    D3DXVECTOR3 m_min;
    D3DXVECTOR3 m_max;
};

//-----------------------------------------------------------------------------
// Name: class CMyD3DApplication
// Desc: Application class. The base class (CD3DApplication) provides the 
//       generic functionality needed in all Direct3D samples. CMyD3DApplication 
//       adds functionality specific to this sample program.
//-----------------------------------------------------------------------------
class CMyD3DApplication : public CD3DApplication
{
    enum e_weight_distribution
    {
        SKIN_WD_SINUSOIDAL,
        SKIN_WD_TRIANGLE,
        SKIN_WD_BRACKETED,
        SKIN_WD_INVERTED_FRACTION,
        SKIN_WD_FRACTION,
        SKIN_WD_SIGMOIDAL
    };
    enum e_transformation
    {
        SKIN_TR_AXIS_ROTATION,
        SKIN_TR_CENTER_SCALING,
        SKIN_TR_Y_TRANSLATE,
        SKIN_TR_AXIS_ROTATION_LEFT,
        SKIN_TR_Y_ROTATION_LEFT,
        SKIN_TR_Y_ROTATION_CENTER
    };
    // true, if the app is loading
    bool m_loading_app;
    CD3DFont *m_font;

    UserInput m_user_input;

    // orientation of the model applied after blending
    float m_rot_x;
    float m_rot_y;

    // mesh object displayed; see d3dfile.cpp
    CD3DMesh *m_mesh;
    DWORD m_num_vertices;
    DWORD m_num_faces;
    CComPtr<IDirect3DVertexBuffer8> m_vertices;
    CComPtr<IDirect3DIndexBuffer8> m_indices;
    s_mesh_bounds m_mesh_bounds;

    // Vertex blending matrices
    D3DXMATRIX m_upper_arm;
    D3DXMATRIX m_lower_arm;

    // Vertex shader
    DWORD m_vertex_shader;
    bool m_use_vertex_shader;

    // options: weight distribution, transformation, stats
    e_weight_distribution m_weight_dist;
    e_transformation m_trans;
    bool m_show_stats;

    // view and projection matrices used
    D3DXMATRIX m_view;
    D3DXMATRIX m_projection;

    // assigns weights to vertices based on m_weight_dist
    void compute_weights();

    // handles option menus
    LRESULT on_wm_command(HWND window, UINT msg, WPARAM wp, LPARAM lp);

protected:
    HRESULT OneTimeSceneInit();
    HRESULT InitDeviceObjects();
    HRESULT RestoreDeviceObjects();
    HRESULT InvalidateDeviceObjects();
    HRESULT DeleteDeviceObjects();
    HRESULT Render();
    HRESULT FrameMove();
    HRESULT FinalCleanup();
    HRESULT ConfirmDevice(D3DCAPS8*, DWORD, D3DFORMAT);

    HRESULT RenderText();

    void UpdateInput(UserInput* pUserInput);

    void ReadSettings();
    void WriteSettings();

public:
    LRESULT MsgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
    CMyD3DApplication();
};

#endif